@@ -5,11 +5,11 @@ from .theplatform import ThePlatformBaseIE
 from ..utils import (
     ExtractorError,
     GeoRestrictedError,
-    int_or_none,
     OnDemandPagedList,
+    int_or_none,
     try_get,
-    urljoin,
     update_url_query,
+    urljoin,
 )
 
 
@@ -73,6 +73,7 @@ class MediasetIE(ThePlatformBaseIE):
             'season_number': 5,
             'episode_number': 5,
             'chapters': [{'start_time': 0.0, 'end_time': 3409.08}, {'start_time': 3409.08, 'end_time': 6565.008}],
+            'categories': ['Informazione'],
         },
     }, {
         # DRM
@@ -127,7 +128,8 @@ class MediasetIE(ThePlatformBaseIE):
         },
         'params': {
             'skip_download': True,
-        }
+        },
+        'skip': 'Dead link',
     }, {
         # WittyTV embed
         'url': 'https://www.wittytv.it/mauriziocostanzoshow/ultima-puntata-venerdi-25-novembre/',
@@ -148,6 +150,7 @@ class MediasetIE(ThePlatformBaseIE):
             'season_number': 12,
             'episode': 'Episode 8',
             'episode_number': 8,
+            'categories': ['Intrattenimento'],
         },
         'params': {
             'skip_download': True,
