@@ -1,8 +1,5 @@
 from .common import InfoExtractor
-from ..utils import (
-    int_or_none,
-    try_get
-)
+from ..utils import int_or_none, try_get
 
 
 class OlympicsReplayIE(InfoExtractor):
